@@ -1,14 +1,13 @@
 #!/usr/bin/perl
 use strict; use warnings;
 
-use lib 'lib';
 use Google::Adwords::AccountService;
 
 # Initialize client accounts in the sandbox
 
 my $email = '';
 my $password = '';
-my $currency = 'INR';
+my $currency = '';
 
 my $service = Google::Adwords::AccountService->new();
 
@@ -16,7 +15,6 @@ my $service = Google::Adwords::AccountService->new();
 $service->email($email)
         ->password($password)
         ->developerToken($email. '++' . $currency)
-        #->api_version('v7')
         ->use_sandbox(1)
         ->debug(1)
 ;
